package com.biganto.visual.roompark.presentation.screen.deals

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.DealsInteractor
import com.biganto.visual.roompark.domain.interactor.ToursInteractor
import com.biganto.visual.roompark.domain.model.DealPreviewModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class DealsScreenPresenter @Inject constructor(
    private val interactor: DealsInteractor,
    private val toursInteractor: ToursInteractor
)
    : BigantoBasePresenter<DealsScreen, DealsScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> DealsScreenViewState.SomeError(e) }


    private val restoreModel = RestoreModel(mutableListOf())

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(DealsScreenViewState.RestoreView(restoreModel))
    }

    override fun bindIntents() {

        val getStatusList = interactor.getStatusList()

        val fetchDeals = interactor.fetchDeals()
            .flatMap { deals ->
                getStatusList
                    .map{
                        List(deals.size) { index ->
                            DealPreviewModel(
                                Pair(deals[index], it)
                            )
                        }
                    }
            }
            .doOnNext { restoreModel.list = it.toMutableList() }
            .map<DealsScreenViewState>(DealsScreenViewState::DealsLoaded)
            .onErrorReturn(::parseError)


        val getDeals = interactor.getDealsApi()
            .flatMap { deals ->
                getStatusList
                    .map{
                        List(deals.size) { index ->
                            DealPreviewModel(
                                Pair(deals[index], it)
                            )
                        }
                    }
            }
            .doOnNext { restoreModel.list = it.toMutableList() }
            .map<DealsScreenViewState>(DealsScreenViewState::DealsLoaded)
            .onErrorReturn(::parseError)


        val onStartTours = intent(DealsScreen::tourCardClicked)
            .flatMap {dealPreview ->
                toursInteractor.getEstateTourList(dealPreview.tourId,dealPreview.estateId)
                .map { DealsScreenViewState.ToursLoaded(it) }
            }

        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                fetchDeals,
                getDeals,
                onStartTours
            )
        )
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(DealsScreenViewState::class.java), DealsScreen::render)
    }
}



data class RestoreModel(
    var list:MutableList<DealPreviewModel>
)